//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <stdio.h>
#include <stdlib.h>
#include <sstream>
#include "city.h"
#include "army.h"
#include "hero.h"
#include "stacklist.h"
#include "playerlist.h"
#include "armysetlist.h"
#include "defs.h"
#include "counter.h"

using namespace std;

//#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl;}
#define debug(x)

City::City(PG_Point pos, string name, int gold)
	:Location(name, pos, 2), d_player(0), d_duration(-1),
	d_production(-1), d_gold(gold), d_burnt(false), d_defense_level(1)
{
    //assign id
    d_id = fl_counter->getNextId();

    // Initialise armytypes
    for (int i = 0; i < 4; i++)
    {
        d_armytype[i].type = -1; 
        d_armytype[i].armyset = "none";
    }
}

City::City(XML_Helper* helper)
	:Location()
{
	d_size = 2;
	
	//initialize the city
	helper->getData(d_id, "id");
	helper->getData(d_name, "name");

	int i;
	helper->getData(i, "x");
    d_pos.x = i;
	helper->getData(i, "y");
    d_pos.y = i;
	helper->getData(i, "owner");
	d_player = Playerlist::getInstance()->getPlayer(static_cast<unsigned int>(i));

	//get production types
	istringstream is;
	string s;

	helper->getData(s, "armytype");
	is.str(s);
	for (i = 0; i < 4; i++)
		is >>d_armytype[i].armyset >>d_armytype[i].type;

	helper->getData(d_duration, "duration");
	helper->getData(d_production, "production");
	helper->getData(d_gold, "gold");
	helper->getData(d_burnt, "burnt");
	helper->getData(d_defense_level, "defense");
}

City::~City()
{
}

void City::nextTurn()
{
    if (d_burnt)
        return;

	// check if an army should be produced
	if (d_production >= 0 && --d_duration == 0) 
	{
		produceArmy();
	}

	// add goldproduction of city to the gold of owner
	d_player->addGold(d_gold);	
}

// Find a stack that isn't full on one of the 4 possible citytiles
// or create a new one if necessary. If there isn't enough room
// for a new stack return 0

Stack* City::getFreeStack()
{
	for (int i = 0; i < 2; i++)
		for (int j = 0; j < 2; j++)
		{
			Stack* stack = Stacklist::getObjectAt(d_pos.x + j, d_pos.y+ i);

			// No stack found so create one
			if (stack == 0)
			{
				PG_Point temp;
				temp.x = d_pos.x + j;
				temp.y = d_pos.y + i;
				stack = new Stack(d_player, temp);
				d_player->addStack(stack);
				return stack;
			}
			else if (stack->size() < 8) return stack;
		}

	return 0;
}

// add a hero to a free stack
void City::addHero(Hero* hero)
{
	Stack* stack = getFreeStack();

	// No stack found so create one
	if (!stack)
	{
		stack = new Stack(d_player, d_pos);
		d_player->addStack(stack);
	}

	// add hero to stack
	stack->push_front(hero);
}

// produce a new army and add it to a free stack
// if there is no free stack set the same production
// again and hope that there is enough space later

void City::produceArmy()
{
	debug("produce_army()\n");
	Stack* stack = getFreeStack();

	if (stack)
	{
		// add produced army to stack
        stack->push_back(new Army(Armysetlist::getInstance()->getArmyset(d_armytype[d_production].armyset),
                                    d_armytype[d_production].type, d_player));
	}
	// start producing next army of same type
	setProduction(d_production);
}

void City::setProduction(int production)
{
	d_production = production;

	if (production >= 0)
	{
		// set the duration to produce this armytype
		d_duration = Armysetlist::getInstance()->getArmyset(d_armytype[d_production].armyset)
                        ->army[d_armytype[d_production].type].getProduction(); 
	}
}

void City::setArmytype(int index, int armytype, Armyset* set)
{
    d_armytype[index].type = armytype;
    if (set)
        d_armytype[index].armyset = set->getName();
    else
        d_armytype[index].armyset = d_player->getArmyset()->getName();

    if (armytype == -1)
    {
        //some production is deleted, so we have to ensure that the data remains
        //consistent. Sort the remaining armytypes and change the production
        //if neccessary
        for (int i = index; i < 3; i++)
            d_armytype[i] = d_armytype[i+1];
        d_armytype[3].type = -1;
        d_armytype[3].armyset = "none";

        if (d_production == index)
            setProduction(-1);
    }
}

bool City::isFriend(Player* player) const
{
    return (d_player == player);
}

// set the possible armyproductions of the city randomly

void City::setRandomArmytypes()
{
    //always set the lowest armytype
    setArmytype(0, 2);

    //ad other armytypes if the dice are lucky
	int random_nr = rand() % 2;
	if (random_nr == 1) 
	{
        setArmytype(1, 3);
		random_nr = rand() % 4;
		if (random_nr == 3)
            setArmytype(2, 4 + (rand() % 3));
	}
}

bool City::hasProduction(int type, Armyset* set)
{
    if (set == 0)
        set = d_player->getArmyset();
    
	for (int i = 0; i < 4; i++)
	{
		if ((d_armytype[i].type == type) && (d_armytype[i].armyset == set->getName()))
            return true;
	}
	return false;
}

void City::produceLargestArmy()
{
	debug("produceLargestArmy()");

    // all production slots empty?
    if (d_armytype[0].type == 0)
    {
        debug("couldn't produce army. no production type available!");
        return;
    }

	Stack* stack = getFreeStack();
	if (stack)
	{
        for (int i = 3; i >= 0; i--)
        {
            if (d_armytype[i].type > 0) {
                // add produced army to stack
                stack->push_back(new Army(Armysetlist::getInstance()->getArmyset(d_armytype[i].armyset),
                                            d_armytype[i].type, d_player));
                return;
            }
        }
    }
}

bool City::save(XML_Helper* helper)
{
	bool retval = true;

	stringstream s;
	for (int i = 0; i < 4; i++)
    {
		s << d_armytype[i].armyset << " " <<d_armytype[i].type <<" ";
    }

	retval &= helper->openTag("city");
	retval &= helper->saveData("id", d_id);
	retval &= helper->saveData("name", d_name);
	retval &= helper->saveData("x", d_pos.x);
	retval &= helper->saveData("y", d_pos.y);
	retval &= helper->saveData("owner", d_player->getId());
	retval &= helper->saveData("duration", d_duration);
	retval &= helper->saveData("armytype", s.str());
	retval &= helper->saveData("production", d_production);
	retval &= helper->saveData("gold", d_gold);
	retval &= helper->saveData("burnt", d_burnt);
	retval &= helper->saveData("defense", d_defense_level);
	retval &= helper->closeTag();
	return retval;
}

// End of file
