//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <paragui.h>
#include <pgapplication.h>
#include <pglog.h>
#include <iostream>
#include <stdlib.h>
#include <time.h>
#include <sys/stat.h>
#include "splash.h"
#include "Configuration.h"
#include "File.h"
#include "GraphicsCache.h"

using namespace std;

SDL_Rect* getBestResolution(Uint32 mymode)
{
	SDL_Rect **modes;
	int i;

	// Get available fullscreen/hardware modes
	modes = SDL_ListModes(NULL, mymode);

	// Check if there are any modes available
	if(modes == (SDL_Rect **)0)
	{
		cerr << "No modes available!\n";
		exit(-1);
	}

	// Check if any resolution is ok
	if(modes == (SDL_Rect **)-1)
	{
		cout << "All resolutions available.\n";
		return 0;
	}
	else
	{
		// Print valid modes
		cout << "\nAvailable Modes\n";

		for(i = 0; modes[i]; ++i)
		{
			cout << "Resolution = " << modes[i]->w << "x" << modes[i]->h << endl;
		}
		if ((i == 0) && (modes[i]->w == 640))
		{
			cout << "You need at least 800 x 600 resolution to run FreeLords!\n";
			exit(-1);
		}
	}

	return modes[0];	// return the highest resolution
}


int main(int argc, char* argv[])
{
	char version[10];
    strcpy(version,FREELORDS_VERSION);
    char theme[20];
    strcpy(theme, "freelords");
	srand(time(NULL));		 // set the random seed

    Configuration conf;

#ifndef __WIN32__
    //the order is: first look in /etc/freelordsrc for a syste-wide file. This
    //should contain global information, most important datapath, since normally
    //this path should be known. Second, we look for a .freelordsrc in the
    //user's home directory. This should contain savepath and, if wanted, some
    //additional information (resolution, fullscreen).

    //What happens now if one or both of the files don't exist? In this case, we
    //assume default values, namely ~/.freelords for the save files and
    ///usr/local/share/freelords for the data files. This may be subject to
    //change, see Configuration.cpp for the actual values.

    conf.loadConfigurationFile("/etc/freelordsrc");

    // now check for ~/.freelordsrc
    // read the environment variable HOME
    char* home = getenv("HOME");
    string path = home;
    path += "/.freelordsrc";
    conf.loadConfigurationFile(path);

    if (Configuration::s_savePath == "")
    {
        //no savepath loaded, so we have to set it to the default
        Configuration::s_savePath = home;
        Configuration::s_savePath +="/.freelords";
    }

#else
    conf.loadConfigurationFile("freelordsrc");
#endif

    //Check if the save game directory exists. If not, try to create it.
    struct stat testdir;

    if (stat(Configuration::s_savePath.c_str(), &testdir)
        || !S_ISDIR(testdir.st_mode))
    {
        Uint32 mask = 07750; //make directory only readable for user and group
        if (mkdir(Configuration::s_savePath.c_str(), mask))
        {
            std::cerr <<"Couldn't create save game directory ";
            std::cerr <<Configuration::s_savePath <<".\n";
            std::cerr <<"Check permissions and the entries in your";
            std::cerr <<"freelordsrc file!\n";
            exit(-1);
        }
    }
    
	if (argc > 1)
	{
		for (int i = 2; i <= argc; i++)
		{
			string parameter(argv[i-1]); 
            if (parameter == "-c")
            {
                i++;
                //convert the next argument
                char* error = 0;
                long size = strtol(argv[i-1], &error, 10);
                if (error && (*error != '\0'))
                {
                    cerr <<"non-numerical value for cache size\n";
                    exit(-1);
                }
                Configuration::s_cacheSize = size;
            }
			if (parameter == "--fullscreen" || parameter == "-f")
			{
                Configuration::s_flags |= SDL_FULLSCREEN;
			}
			if (parameter == "-r1280")
			{
                Configuration::s_width = 1280;
                Configuration::s_height = 1024;
			}
			if (parameter == "-r1024")
			{
                Configuration::s_width = 1024;
                Configuration::s_height = 786;
			}
			if (parameter == "-r800")
			{
                Configuration::s_width = 800;
                Configuration::s_height = 600;
			}
			if (parameter == "-t" || parameter == "--test")
			{
                Configuration::s_testing = true;
			}
			if (parameter == "--help" || parameter == "-h")
			{
				cout << endl;
				cout << "FreeLords " << version << endl << endl;
				cout << "Available parameters:" << endl << endl; 
				cout << "-h,      --help		     Shows this help screen\n";
				cout << "-r1280                      Resolution set to 1280x1024" << endl;
				cout << "-r1024                      Resolution set to 1024x786" << endl;
				cout << "-r800                       Resolution set to 800x600" << endl;
                cout << "-f,      --fullscreen	     Fullscreen mode" << endl;
                cout << "-c <size>                   set the maximum cache size" <<endl;
				cout << "-t,      --test	         Starting with a test-scenario" << endl << endl << endl;
				cout << "Please send bug reports to ulf82@users.sf.net" << endl << endl;
				exit(0);
			}
		}
	}

    //quick hack: set the log level to a low niveau. Background: I often run
    //paragui with --enable-debug, but then the console is flooded with
    //debug messages. To prevent this, I decrease the log level.
    PG_LogConsole::SetLogLevel(PG_LOG_MSG);
    
	PG_Application app;
    //do we really want to quit the application by accidentally hitting
    //the Escape key?
//	app.SetEmergencyQuit(true);
    app.SetEmergencyQuit(false);
	
	if (!app.LoadTheme(theme, true, Configuration::s_dataPath.c_str()))
	{
        cerr << "Unable to load theme: " << theme << endl;
        exit(-1);
	}
	
    // Check if armysets are in the path (otherwise exit)
    File::scanArmysets();

    app.SetFontSize(14);

    // if resolution hasn't been set explicitly in the configuration file
    // or on the commandline guess the best resolution
    if (Configuration::s_width == -1)
    {
        cout << "no resolution provided: guessing" << endl;
        SDL_Rect* mode = getBestResolution(Configuration::s_flags &
                                            Configuration::s_surfaceFlags);

        // take the best mode
        if (mode != 0)
        {
            Configuration::s_width = mode->w;
            Configuration::s_height = mode->h;
       	}
        // couldn't guess a mode - most probably windowed mode
        // set minimum of 800x600
        else
        {
            Configuration::s_width = 800;
            Configuration::s_height = 600;
        }
   	}   	

    cout << "Using resoluting: " << Configuration::s_width << "x" << Configuration::s_height << endl;	
	
	if (!app.InitScreen(Configuration::s_width, Configuration::s_height, 16,
                        Configuration::s_flags & Configuration::s_surfaceFlags))
	{
		cerr << "Couldn't initialize X Windowing System\n";
		exit(1);
	}

	char title[50];
	sprintf(title, "FreeLords %s", version);
		
	SDL_WM_SetCaption(title, "");

	// Initialize the SDL Timer sub-system
	SDL_InitSubSystem(SDL_INIT_TIMER);

	Splash splash(&app, PG_Rect(0, 0, Configuration::s_width, Configuration::s_height));
	splash.Show();
	
	if (Configuration::s_testing)
	{
        splash.newGame(GameScenario::SINGLE_PLAYER);
    }

	app.Run();

	return EXIT_SUCCESS;
}
