//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef PLAYER_H
#define PLAYER_H

#include <string>
#include <list>
#include <pgmessageobject.h>
#include "defs.h"
#include "fight.h"
#include "action.h"
#include "city.h"
#include "armyset.h"
#include "xmlhelper.h"
#include <sigc++/sigc++.h>

/* the basic player class, includes all functionality which all players have
 * either in common or which is required by other objects whih don't need to
 * know which player this actually is

 * some notes to the player derivation scheme:
 * The derivation looks like this:
 * 
 * Player
 *   |
 *   +--- NetworkPlayer (NIY)
 *   |
 *   +--- RealPlayer (a local player)
 *          +
 *          |
 *          +--- AI_Dummy
 *          |
 *          +--- AI_Fast
 *
 * The typical player class is the RealPlayer. It includes all the stuff which
 * a player can do (moving armies, buying city's production etc.). The normal
 * human player is represented by a ReaPlayer. AI players use the same
 * functionality, but overwrite the virtual functions start_turn and end_turn.
 * start_turn is called at the start of the player's turn, while end_turn is
 * intended to clean up any mess the player may do. Since the RealPlayer hardly
 * shares anything functional with network players (their whole business of e.g.
 * moving armies is done differently), who are from the other object's point of
 * view just other players, I have created an abstract class player for this
 * purpose.
 */

class Stacklist;
class Hero;

class Player : public SigC::Object
{
	public:
		enum TYPE {HUMAN = 0, AI_FAST = 1, AI_DUMMY = 2, NEUTRAL = 3};
		
		// CREATORS
		Player (std::string name, std::string armyset, SDL_Color color,
			TYPE type);
		Player(const Player&);
		Player(XML_Helper* helper);
		virtual ~Player();
		
		// MANIPULATORS
		void setArmyset(std::string armyset){d_armyset = armyset;}
		void setDead(bool dead){d_dead = dead;}
		void setType(TYPE type) {d_type = type;}
		void addGold(int gold);
		void withdrawGold(int gold);
		void clearActionlist();
        void addStack(Stack* stack);
        bool deleteStack(Stack* stack);
		
		// ACCESSORS
		unsigned int    getId(){return d_id;}
		std::list<Action*>  getActionlist(){return d_actions;}
		Armyset*        getArmyset();
		TYPE            getType() {return d_type;}
		std::string     getArmysetName(){return d_armyset;}
		SDL_Color       getColor() const {return d_color;}
        SDL_Color*      getPalette() {return d_palette;}
		int             getGold(){return d_gold;}
		std::string     getName(){return d_name;}
        Stacklist*      getStacklist(){return d_stacklist;}
        //this is nothing more than a shortcut
        Stack*          getActivestack();
        
        //misc other functions
        void initPalette();
		void dumpActionlist();
		bool enoughMoves(Stack* s);
		
		//Save/Load
		bool save(XML_Helper* helper);
		static Player* loadPlayer(XML_Helper* helper);
			//this is a little bit inconsistent, but there are many
			//types of players and somewhere has to be done a
			//switch(type){...}, so why not here?
		bool load(std::string tag, XML_Helper* helper);
			//load actions
		
		//ACTIONS
		virtual bool startTurn() = 0;

        /** Called so that the player can decide what to do with an
            occupied city as soon as he has occupied it (mainly for AI)

            @return always returns true (unused feature)
            @param  city   the conquered city
          */
        virtual bool invadeCity(City* c) = 0;

        /** Called whenever a hero decides to join you, so the player can
            decide what to do with the hero.

            @return true if the player accepts the hero, false otherwise
            @param  hero    the hero who has offered his service
            @param  cost    the amount of gold neccessary to recruit the
                            hero

            \note Don't touch the hero except for naming purposes etc.
                  The hero is integrated elsewhere!
          */
        virtual bool recruitHero(Hero* hero, int cost) = 0;
        
		virtual bool stackSplit(Stack* s) =0;
		virtual bool stackJoin(Stack* receiver, Stack* joining) =0;

		virtual bool stackMove(Stack* s) =0;
		virtual bool stackMove(Stack* s, PG_Point dest, bool follow)=0;
		
		virtual Fight::RESULT stackFight(Stack* sattacker, Stack* sdefender) =0;
		virtual Fight::RESULT stackFightCity(std::vector<Stack*> attackers, City* target) =0;

		virtual bool stackSearchRuin(Stack* s, Ruin* r) =0;
		virtual bool stackVisitTemple(Stack* s, Temple* t) =0;

		virtual bool cityOccupy(City* c) =0;
        virtual bool cityPillage(City* c) = 0;
		virtual bool cityRaze(City* c) =0;

		virtual bool cityUpgradeDefense(City* c) =0;
		virtual bool cityBuyProduction(City* c, int armytype) =0;
		virtual bool cityChangeProduction(City* c, int production) =0;

        //finally, some signals
        SigC::Signal1<void, City*> sinvadingCity;
        SigC::Signal2<bool, Hero*, int> srecruitingHero;
        
        SigC::Signal0<void> schangingStatus;
        SigC::Signal1<void, Stack*> supdatingStack;
        SigC::Signal1<void, City*> supdatingCity;
        SigC::Signal0<void> sinterruptTimers;
        SigC::Signal0<void> scontinueTimers;

        //if reward != 0, give this reward, else randomize
		virtual bool giveReward (int level, Stack* s, fl_Treasure* reward = 0) = 0;
        
	protected:
		virtual bool stackMoveOneStep(Stack* s) = 0;

		// DATA
		SDL_Color d_color;
        SDL_Color* d_palette;
		std::string d_name;
		std::string d_armyset;	// name of the armyset
		int d_gold;
		bool d_dead;		    // is player already dead?
		TYPE d_type;
		unsigned int d_id;
		std::list<Action*> d_actions; //list of actions done by the player
        Stacklist* d_stacklist;
};

extern SigC::Signal1<void, Player::TYPE>  sendingTurn;

#endif // PLAYER_H

// End of file
